@@ -23,7 +23,6 @@ import os
 import sys
 from time import strftime
 import copy
-import random
 import getpass
 
 #related third party imports
@@ -37,9 +36,7 @@ mpl.use("AGG")
 from matplotlib import pyplot as plt
 from matplotlib import colors as mpl_colors
 from matplotlib import cm as mpl_cm
-from matplotlib.ticker import FormatStrFormatter,ScalarFormatter,AutoMinorLocator
-
-from scipy import stats
+from matplotlib.ticker import ScalarFormatter
 
 #local application/library specific imports
 import lalinference.bayespputils as bppu
@@ -50,7 +47,7 @@ __version__= "git id %s"%git_version.id
 __date__= git_version.date
 
 #List of parameters to plot/bin . Need to match (converted) column names.
-oneDMenu=['mtotal','m1','m2','mchirp','mc','chirpmass','distance','distMPC','dist','iota','psi','eta','q','asym_massratio','spin1','spin2','a1','a2','phi1','theta1','phi2','theta2','costilt1','costilt2','costhetas','cosbeta','phi_orb', 'lambdat', 'dlambdat', 'lambda1', 'lambda2', 'lam_tilde', 'dlam_tilde','theta_jn','a1z','a2z'] + bppu.snrParams + bppu.spinParams + bppu.cosmoParam + bppu.calParams
+oneDMenu=['mtotal','m1','m2','mchirp','mc','chirpmass','distance','distMPC','dist','iota','psi','eta','q','asym_massratio','spin1','spin2','a1','a2','phi1','theta1','phi2','theta2','costilt1','costilt2','costhetas','cosbeta','phi_orb', 'lambdat', 'dlambdat', 'lambda1', 'lambda2', 'lam_tilde', 'dlam_tilde','theta_jn','a1z','a2z'] + bppu.snrParams + bppu.spinParams + bppu.cosmoParam + bppu.calParams + bppu.lorentzInvarianceViolationParams
 #List of parameter pairs to bin . Need to match (converted) column names.
 twoDGreedyMenu=[['mc','eta'],['mchirp','eta'],['chirpmass','eta'],['mc','q'],['mchirp','q'],['chirpmass','q'],['mc','asym_massratio'],['mchirp','asym_massratio'],['chirpmass','asym_massratio'],['m1','m2'],['mtotal','eta'],['distance','iota'],['dist','iota'],['dist','m1'],['ra','dec'],['dist','cos(iota)'],['phi_orb','iota'],['theta_jn','dist'],['spin1','spin2'],['spin1','mchirp'],['spin1','m1'],['a1','a2'],['a1','mchirp'],['a1','m1'],['tilt1','tilt2'],['tilt1','mchirp'],['tilt1','m1'],['a1z','a2z']]
 #Bin size/resolution for binning. Need to match (converted) column names.
@@ -65,10 +62,10 @@ paramNameLatexMap = {'m1': 'm_1', 'm2' : 'm_2', 'mtotal' : r'M_{\rm tot}', 'mchi
                      'cos(iota)': r'\cos \iota', 'tilt1': r't_1', 'tilt2': r't_2', 'ra': r'\alpha', 'dec': r'\delta',
                      'lambdat' : r'\tilde{\Lambda}', 'dlambdat': r'\delta \tilde{\Lambda}',
                      'lambda1' : r'\lambda_1', 'lambda2': r'\lambda_2',
-                     'lam_tilde' : r'\tilde{\Lambda}', 'dlam_tilde': r'\delta \tilde{\Lambda}','dchi0':r'\delta\chi_0','dchi1':r'\delta\chi_1','dchi2':r'\delta\chi_2','dchi3':r'\delta\chi_3','dchi4':r'\delta\chi_4','dchi5':r'\delta\chi_5','dchi5l':r'\delta\chi_{5l}','dchi6':r'\delta\chi_6','dchi6l':r'\delta\chi_{6l}','dchi7':r'\delta\chi_7','dbeta2':r'\delta\beta_2','dbeta3':r'\delta\beta_3','dsigma2':r'\delta\sigma_2','dsigma3':r'\delta\sigma_3','dsigma4':r'\delta\sigma_4','dbeta2':r'\delta\beta_2','dbeta3':r'\delta\beta_3' }
+                     'lam_tilde' : r'\tilde{\Lambda}', 'dlam_tilde': r'\delta \tilde{\Lambda}','dchi0':r'\delta\chi_0','dchi1':r'\delta\chi_1','dchi2':r'\delta\chi_2','dchi3':r'\delta\chi_3','dchi4':r'\delta\chi_4','dchi5':r'\delta\chi_5','dchi5l':r'\delta\chi_{5l}','dchi6':r'\delta\chi_6','dchi6l':r'\delta\chi_{6l}','dchi7':r'\delta\chi_7','dbeta2':r'\delta\beta_2','dbeta3':r'\delta\beta_3','dsigma2':r'\delta\sigma_2','dsigma3':r'\delta\sigma_3','dsigma4':r'\delta\sigma_4','dbeta2':r'\delta\beta_2','dbeta3':r'\delta\beta_3' ,'log10lambda_a':r'$\log\lambda_{\mathbb{A}}$','log10lambda_eff':r'$\log\lambda_{eff}$','log10livamp':r'$\log \mathbb{A}$','lambda_a':r'$\lambda_{\mathbb{A}}$','lambda_eff':r'$\lambda_{eff}$','liv_amp':r'$\mathbb{A}$'}
 
 # Only these parameters, in this order appear in confidence level table.
-clTableParams = ['mchirp', 'mc', 'chirpmass', 'eta', 'q', 'm1', 'm2', 'distance', 'distMPC', 'dist', 'cos(iota)', 'iota', 'theta_jn', 'psi', 'ra', 'dec', 'time', 'phase', 'a1', 'a2', 'costilt1', 'costilt2','dchi0','dchi1','dchi2','dchi3','dchi4','dchi5','dchi5l','dchi6','dchi6l','dchi7','dbeta2','dbeta3','dsigma2','dsigma3','dsigma4','dbeta2','dbeta3']
+clTableParams = ['mchirp', 'mc', 'chirpmass', 'eta', 'q', 'm1', 'm2', 'distance', 'distMPC', 'dist', 'cos(iota)', 'iota', 'theta_jn', 'psi', 'ra', 'dec', 'time', 'phase', 'a1', 'a2', 'costilt1', 'costilt2','dchi0','dchi1','dchi2','dchi3','dchi4','dchi5','dchi5l','dchi6','dchi6l','dchi7','dbeta2','dbeta3','dsigma2','dsigma3','dsigma4','dbeta2','dbeta3', 'log10lambda_eff','log10lambda_a','log10livamp','lambda_eff','lambda_a','liv_amp']
 
 
 greedyBinSizes={'mc':0.001,'m1':0.1,'m2':0.1,'mass1':0.1,'mass2':0.1,'mtotal':0.1,'eta':0.001,'q':0.001,'asym_massratio':0.001,'iota':0.05,'time':1e-4,'distance':5.0,'dist':1.0,'mchirp':0.01,'chirpmass':0.01,'a1':0.02,'a2':0.02,'phi1':0.05,'phi2':0.05,'theta1':0.05,'theta2':0.05,'ra':0.05,'dec':0.005,'psi':0.1,'cos(iota)':0.01, 'cos(tilt1)':0.01, 'cos(tilt2)':0.01, 'tilt1':0.05, 'tilt2':0.05, 'cos(thetas)':0.01, 'cos(beta)':0.01,'phi_orb':0.2,'inclination':0.05,'theta_jn':0.05,'spin1':0.02,'spin2':0.02}
@@ -76,7 +73,7 @@ for s in bppu.snrParams:
         greedyBinSizes[s]=0.02
 for s in bppu.calParams:
         greedyBinSizes[s]=0.02
-for s in bppu.tigerParams:
+for s in bppu.tigerParams + bppu.lorentzInvarianceViolationParams:
   greedyBinSizes[s]=0.01
 for s in bppu.spinParams:
         greedyBinSizes[s]=0.02
@@ -144,7 +141,6 @@ def all_pairs(L):
 
 def open_url_curl(url,args=[]):
     import subprocess
-    import urlparse
 
     kerberos_args = "--insecure -c /tmp/{0}_cookies -b /tmp/{0}_cookies --negotiate --user : --location-trusted".format(getpass.getuser()).split()
 
@@ -175,13 +171,11 @@ def compare_plots_one_param_pdf(list_of_pos_by_name,param,analyticPDF=None):
 
     """
 
-    from scipy import seterr as sp_seterr
-
     #Create common figure
     myfig=plt.figure(figsize=(6,4.5),dpi=150)
 
-    list_of_pos=list_of_pos_by_name.values()
-    list_of_pos_names=list_of_pos_by_name.keys()
+    list_of_pos=list(list_of_pos_by_name.values())
+    list_of_pos_names=list(list_of_pos_by_name.keys())
 
     allmins=map(lambda a: np.min(a[param].samples), list_of_pos)
     allmaxes=map(lambda a: np.max(a[param].samples), list_of_pos)
@@ -232,7 +226,7 @@ def compare_plots_one_param_pdf(list_of_pos_by_name,param,analyticPDF=None):
             if min(pos_samps)<injpar and max(pos_samps)>injpar:
                 plt.plot([injpar,injpar],[0,max(kdepdf)],'r-.',scalex=False,scaley=False)
     if analyticPDF is not None:
-	plt.plot(ind,map(analyticPDF,ind),'r')
+        plt.plot(ind,map(analyticPDF,ind),'r')
     #
     return myfig#,rkde
 #
@@ -259,8 +253,6 @@ def compare_plots_one_param_line_hist(list_of_pos_by_name,param,cl,color_by_name
 
     """
 
-    from scipy import seterr as sp_seterr
-
     #Create common figure
     myfig=plt.figure(figsize=(6,4.5),dpi=150)
   #myfig.add_axes([0.1,0.1,0.65,0.85])
@@ -274,8 +266,8 @@ def compare_plots_one_param_line_hist(list_of_pos_by_name,param,cl,color_by_name
     majorFormatterX.set_scientific(True)
     majorFormatterY.set_scientific(True)
 
-    list_of_pos=list_of_pos_by_name.values()
-    list_of_pos_names=list_of_pos_by_name.keys()
+    list_of_pos=list(list_of_pos_by_name.values())
+    list_of_pos_names=list(list_of_pos_by_name.keys())
 
     allmins=map(lambda a: np.min(a[param].samples), list_of_pos)
     allmaxes=map(lambda a: np.max(a[param].samples), list_of_pos)
@@ -296,16 +288,16 @@ def compare_plots_one_param_line_hist(list_of_pos_by_name,param,cl,color_by_name
             injvals.append(posterior[param].injval)
 
         try:
-            n,bins=np.histogram(posterior[param].samples,bins=posbins,normed=True,new=True)
+            n,bins=np.histogram(posterior[param].samples,bins=posbins,density=True,new=True)
         except:
-            n,bins=np.histogram(posterior[param].samples,bins=posbins,normed=True)
+            n,bins=np.histogram(posterior[param].samples,bins=posbins,density=True)
         if min(bins)==max(bins):
             print('Skipping '+param)
             continue
         locmaxy=max(n)
         if locmaxy>max_y: max_y=locmaxy
 #(n, bins, patches)=plt.hist(posterior[param].samples,bins=bins,facecolor='white',label=name,normed=True,hold=True,color=color_by_name[name])#range=(min_pos,max_pos)
-        (n, bins, patches)=plt.hist(posterior[param].samples,bins=bins,histtype='step',label=name,normed=True,hold=True,color=color_by_name[name])
+        (n, bins, patches)=plt.hist(posterior[param].samples,bins=bins,histtype='step',label=name,density=True,hold=True,color=color_by_name[name])
         patch_list.append(patches[0])
 
     Nchars=max(map(lambda d:len(majorFormatterX.format_data(d)),axes.get_xticks()))
@@ -323,7 +315,7 @@ def compare_plots_one_param_line_hist(list_of_pos_by_name,param,cl,color_by_name
 
     plt.xlim(min_pos,max_pos)
     top_cl_intervals_list={}
-    pos_names=list_of_pos_by_name.keys()
+    pos_names=list(list_of_pos_by_name.keys())
 
 
     for name,posterior in list_of_pos_by_name.items():
@@ -360,7 +352,7 @@ def compare_plots_one_param_line_hist(list_of_pos_by_name,param,cl,color_by_name
 
     #
     if analyticPDF is not None:
-	plt.plot(posbins,map(analyticPDF,posbins),'r')
+        plt.plot(posbins,map(analyticPDF,posbins),'r')
     return myfig,top_cl_intervals_list#,rkde
 
 #
@@ -386,7 +378,7 @@ def compute_ks_pvalue_matrix(list_of_pos_by_name, param):
             matrix[j,i]=pvalue
 
     return matrix
-        
+
 def compare_plots_one_param_line_hist_cum(list_of_pos_by_name,param,cl,color_by_name,cl_lines_flag=True,analyticCDF=None,legend='auto'):
 
     """
@@ -409,20 +401,18 @@ def compare_plots_one_param_line_hist_cum(list_of_pos_by_name,param,cl,color_by_
 
     """
 
-    from scipy import seterr as sp_seterr
-
     #Create common figure
     myfig=plt.figure(figsize=(6,4.5),dpi=150)
     myfig.add_axes([0.15,0.15,0.6,0.76])
-    list_of_pos=list_of_pos_by_name.values()
-    list_of_pos_names=list_of_pos_by_name.keys()
+    list_of_pos=list(list_of_pos_by_name.values())
+    list_of_pos_names=list(list_of_pos_by_name.keys())
 
     injvals=[]
     allmins=map(lambda a: np.min(a[param].samples), list_of_pos)
     allmaxes=map(lambda a: np.max(a[param].samples), list_of_pos)
     min_pos=np.min(allmins)
     max_pos=np.max(allmaxes)
- 
+
     patch_list=[]
     max_y=1.
 
@@ -435,20 +425,20 @@ def compare_plots_one_param_line_hist_cum(list_of_pos_by_name,param,cl,color_by_
             injvals.append(posterior[param].injval)
 
         try:
-            n,bins=np.histogram(posterior[param].samples,bins=posbins,normed=True,new=True)
+            n,bins=np.histogram(posterior[param].samples,bins=posbins,density=True,new=True)
         except:
-            n,bins=np.histogram(posterior[param].samples,bins=posbins,normed=True)
+            n,bins=np.histogram(posterior[param].samples,bins=posbins,density=True)
 
         if min(bins)==max(bins):
             print('Skipping '+param)
             continue
 
-        (n, bins, patches)=plt.hist(posterior[param].samples,bins=bins,histtype='step',label=name,normed=True,hold=True,color=color_by_name[name],cumulative='True')#range=(min_pos,max_pos)
+        (n, bins, patches)=plt.hist(posterior[param].samples,bins=bins,histtype='step',label=name,density=True,hold=True,color=color_by_name[name],cumulative='True')#range=(min_pos,max_pos)
 
         patch_list.append(patches[0])
 
     top_cl_intervals_list={}
-    pos_names=list_of_pos_by_name.keys()
+    pos_names=list(list_of_pos_by_name.keys())
 
 
     for name,posterior in list_of_pos_by_name.items():
@@ -484,7 +474,7 @@ def compare_plots_one_param_line_hist_cum(list_of_pos_by_name,param,cl,color_by_
         #if min(pos_samps)<injpar and max(pos_samps)>injpar:
         plt.plot([injpar,injpar],[0,max_y],'r-.',scalex=False,scaley=False,linewidth=4,label='Injection')
     if analyticCDF is not None:
-	plt.plot(posbins,map(analyticCDF,posbins),'r')
+        plt.plot(posbins,map(analyticCDF,posbins),'r')
     return myfig,top_cl_intervals_list#,rkde
 
 
@@ -494,7 +484,6 @@ def compare_bayes(outdir,names_and_pos_folders,injection_path,eventnum,username,
 
     if injection_path is not None and os.path.exists(injection_path) and eventnum is not None:
         eventnum=int(eventnum)
-        import itertools
         from glue.ligolw import ligolw, lsctables, utils
         injections = lsctables.SimInspiralTable.get_table(
                 utils.load_filename(injection_path, contenthandler = lsctables.use_in(ligolw.LIGOLWContentHandler)))
@@ -504,11 +493,11 @@ def compare_bayes(outdir,names_and_pos_folders,injection_path,eventnum,username,
                 sys.exit(1)
             else:
                 injection=injections[eventnum]
-    
+
     #Create analytic likelihood functions if covariance matrices and mean vectors were given
     analyticLikelihood = None
     if covarianceMatrices and meanVectors:
-	analyticLikelihood = bppu.AnalyticLikelihood(covarianceMatrices, meanVectors)
+        analyticLikelihood = bppu.AnalyticLikelihood(covarianceMatrices, meanVectors)
     peparser=bppu.PEOutputParser('common')
     pos_list={}
     tp_list={}
@@ -691,7 +680,6 @@ def compare_bayes(outdir,names_and_pos_folders,injection_path,eventnum,username,
     print("Common parameters are %s"%str(common_params))
 
     if injection is None and injection_path is not None:
-        import itertools
         injections = SimInspiralUtils.ReadSimInspiralFromFiles([injection_path])
         injection=bppu.get_inj_by_time(injections,pos_temp.means['time'])
     if injection is not None:
@@ -783,13 +771,12 @@ def compare_bayes(outdir,names_and_pos_folders,injection_path,eventnum,username,
             cl_table_min_max_str='<tr><td> Min | Max </td>'
             level_index=0
             for confidence_level in OneDconfidenceLevels:
-		if analyticLikelihood:
-		  pdf=analyticLikelihood.pdf(param)
-		  cdf=analyticLikelihood.cdf(param)
-		else:
-		  pdf=None
-		  cdf=None
-		  
+                if analyticLikelihood:
+                    pdf=analyticLikelihood.pdf(param)
+                    cdf=analyticLikelihood.cdf(param)
+                else:
+                    pdf=None
+                    cdf=None
                 cl_table_header+='<th colspan="2">%i%% (Lower|Upper)</th>'%(int(100*confidence_level))
                 hist_fig,cl_intervals=compare_plots_one_param_line_hist(pos_list,param,confidence_level,color_by_name,cl_lines_flag=clf,legend=ldg,analyticPDF=pdf)
                 hist_fig2,cl_intervals=compare_plots_one_param_line_hist_cum(pos_list,param,confidence_level,color_by_name,cl_lines_flag=clf,analyticCDF=cdf,legend=ldg)
@@ -797,13 +784,13 @@ def compare_bayes(outdir,names_and_pos_folders,injection_path,eventnum,username,
                 # Save confidence levels and uncertainty
                 #confidence_levels[param]=[]
                 confidence_levels[level_index][param]=[]
-                
+
                 for name,pos in pos_list.items():
                     median=pos[param].median
                     low,high=cl_intervals[name]
                     #confidence_levels[param].append((name,low,median,high))
                     confidence_levels[level_index][param].append((name,low,median,high))
-                    
+
                 level_index=level_index+1
                 cl_bounds=[]
                 poses=[]
@@ -882,7 +869,7 @@ def compare_bayes(outdir,names_and_pos_folders,injection_path,eventnum,username,
             ks_table_str+='</table>'
 
             oned_data[param]=(save_paths,cl_table_str,ks_table_str,cl_uncer_str)
-            
+
     # Watch out---using private variable _logL
     max_logls = [[name,max(pos._logL)] for name,pos in pos_list.items()]
     dics = [pos.DIC for name, pos in pos_list.items()]
@@ -893,7 +880,7 @@ def output_confidence_levels_tex(clevels,outpath):
     """Outputs a LaTeX table of parameter and run medians and confidence levels."""
     outfile=open(os.path.join(outpath,'confidence_table.tex'), 'w')
     for level_index in range(len(OneDconfidenceLevels)):
-        params=clevels[level_index].keys()
+        params=list(clevels[level_index].keys())
 
         clevels_by_name={}
         for param in clTableParams:
@@ -935,7 +922,7 @@ def output_confidence_levels_dat(clevels,outpath):
     """Outputs a LaTeX table of parameter and run medians and confidence levels."""
     outfile=open(os.path.join(outpath,'confidence_table.dat'), 'w')
     for level_index in range(len(OneDconfidenceLevels)):
-        params=clevels[level_index].keys()
+        params=list(clevels[level_index].keys())
 
         clevels_by_name={}
         for param in clTableParams:
@@ -952,7 +939,7 @@ def output_confidence_levels_dat(clevels,outpath):
                 if param in params:
                     tparam=paramNameLatexMap.get(param,param)
                     outfile.write('%s\t'%param)
-            outfile.write('\n') 
+            outfile.write('\n')
 
             for name,levels in clevels_by_name.items():
                 outfile.write(name)
@@ -967,8 +954,8 @@ def output_confidence_levels_dat(clevels,outpath):
 def output_confidence_uncertainty(cluncertainty, outpath):
     outfile=open(os.path.join(outpath, 'confidence_uncertainty.dat'), 'w')
     try:
-        params=cluncertainty.keys()
-        uncer=cluncertainty.values()
+        params=list(cluncertainty.keys())
+        uncer=list(cluncertainty.values())
 
         outfile.write('# Uncertainty in confidence levels.\n')
         outfile.write('# First row is relative uncertainty (wrt to parameter mean).\n')
@@ -988,7 +975,7 @@ def output_confidence_uncertainty(cluncertainty, outpath):
         np.savetxt(outfile, np.reshape(quants, (1,-1)))
     finally:
         outfile.close()
-            
+
 if __name__ == '__main__':
     from optparse import OptionParser
     parser=OptionParser()
@@ -1012,7 +999,7 @@ if __name__ == '__main__':
     parser.add_option("-m","--meanVectors",dest="meanVectors",action="append",default=None,help="Comma separated list of locations of the multivariate gaussian described by the correlation matrix.  First line must be list of params in the order used for the covariance matrix.  Provide one list per covariance matrix.")
     parser.add_option("--no2D",dest="no2d",action="store_true",default=False,help="Disable 2D plots")
     parser.add_option("--npixels-2d",dest="npixels_2d",action="store",type="int",default=50,help="Number of pixels on a side of the 2D plots (default 50)",metavar="N")
-    
+
     (opts,args)=parser.parse_args()
 
     if opts.outpath is None:
@@ -1038,7 +1025,7 @@ if __name__ == '__main__':
 
     # Sort inputs alphabetically
     names,pos_list = zip(*sorted(zip(names,opts.pos_list)))
-    
+
     if opts.no2d:
         twoDplots=[]
 
@@ -1046,9 +1033,9 @@ if __name__ == '__main__':
     greedy2savepaths,oned_data,confidence_uncertainty,confidence_levels,max_logls,dics=compare_bayes(outpath,zip(names,pos_list),opts.inj,opts.eventnum,opts.username,opts.password,opts.reload_flag,opts.clf,opts.ldg_flag,contour_figsize=(float(opts.cw),float(opts.ch)),contour_dpi=int(opts.cdpi),contour_figposition=[0.15,0.15,float(opts.cpw),float(opts.cph)],fail_on_file_err=not opts.readFileErr,covarianceMatrices=opts.covarianceMatrices,meanVectors=opts.meanVectors,Npixels2D=int(opts.npixels_2d))
 
     ####Print Confidence Levels######
-    output_confidence_levels_tex(confidence_levels,outpath)    
+    output_confidence_levels_tex(confidence_levels,outpath)
     output_confidence_levels_dat(confidence_levels,outpath)
-    
+
     ####Save confidence uncertainty#####
     output_confidence_uncertainty(confidence_uncertainty,outpath)
 
