@@ -33,6 +33,3 @@ for _, module, _ in pkgutil.iter_modules([os.path.dirname(__file__)]):
 
 # Clean up
 del os, pkgutil, six
-
-from lalinference.bayestar.deprecation import warn
-warn('ligo.skymap.plot')
