@@ -438,7 +438,7 @@ class OmegaScansDAG(pipeline.CondorDAG):
           timeslides[ifo] = info[ifo][n]
           self.add_node(node)
         self.add_table_entry(info['trigtime'][n],timeslides,self.ifos,0.0)
-    print("succes!")
+    print("success!")
 
     self.write_table()
 
@@ -687,4 +687,4 @@ def main():
 if __name__ == "__main__":
 	# START THE MAIN FUNCTION IF RUN AS A SCRIPT. OTHERWISE, JUST LOADS THE CLASS
 	# AND FUNCTION DEFINITIONS
-	exit(main())
\ No newline at end of file
+	exit(main())
